==> castor_extractor-0.10.0.tar.gz: castor_extractor-0.10.0/castor_extractor/warehouse/snowflake/client.py <==
import logging
import re
from enum import Enum
from typing import Optional

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from sqlalchemy.engine import Connection

from ...utils import to_string_array, uri_encode
from ..abstract import SqlalchemyClient

SNOWFLAKE_URI = "snowflake://{user}:{password}@{account}/?application=castor"
PK_URI = "snowflake://{user}:@{account}/?application=castor"
PUBLIC_ROLE = "PUBLIC"

logger = logging.getLogger(__name__)


def _scalar(connection: Connection, query: str) -> Optional[str]:
    """returns the first column of the first row, None if no row"""
    scalar = connection.execute(query).scalar()
    return None if scalar is None else str(scalar)


class UseResource(Enum):
    """resources using the USE syntax"""

    ROLE = "ROLE"
    WAREHOUSE = "WAREHOUSE"


def _use(connection: Connection, resource: UseResource, name: str):
    """
    USE ROLE/WAREHOUSE do not accept SQL parameter substitution
        we do a sanity check prior to using it
    DON'T COPY THIS CODE FOR ANOTHER PURPOSE
    """
    target = resource.value
    if not re.fullmatch(r"[\w_-]+", name):
        raise NameError(f"{target} name invalid {name}")
    command = f'USE {target} "{name}"'
    logger.info(f"executing {command}")
    connection.execute(command)


def _first_non_public_role(connection: Connection) -> str:
    """fetch the first non public role granted to the user"""
    raw = _scalar(connection, "SELECT CURRENT_AVAILABLE_ROLES()")
    if not raw:
        raise ConnectionError("error at fetching granted roles")
    logger.info(f"available roles {raw}")
    parsed = to_string_array(raw)
    granted = {r.upper() for r in parsed} - {PUBLIC_ROLE}
    if not granted:
        raise ConnectionError("missing non public role")
    return next(iter(granted))


def _private_key(private_key: Optional[str]) -> Optional[bytes]:
    if not private_key:
        return None

    p_key = serialization.load_pem_private_key(
        bytes(private_key, "utf-8"), password=None, backend=default_backend()
    )

    pkb = p_key.private_bytes(
        encoding=serialization.Encoding.DER,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )

    return pkb


class SnowflakeClient(SqlalchemyClient):
    """snowflake client"""

    # hide information message from the connectors
    logging.getLogger("snowflake.connector").setLevel(logging.WARNING)

    def __init__(
        self,
        credentials: dict,
        warehouse: Optional[str] = None,
        role: Optional[str] = None,
    ):
        self.private_key = _private_key(credentials.get("private_key"))
        super().__init__(credentials)
        self._role = role
        self._warehouse = warehouse

    @staticmethod
    def name() -> str:
        return "Snowflake"

    def _engine_options(self, credentials: dict) -> dict:
        if not self.private_key:
            return {}
        return {
            "connect_args": {
                "private_key": self.private_key,
            }
        }

    def _build_uri(self, credentials: dict) -> str:
        if self.private_key:
            return PK_URI.format(
                account=credentials["account"],
                user=credentials["user"],
            )
        return SNOWFLAKE_URI.format(
            user=credentials["user"],
            password=uri_encode(credentials["password"]),
            account=credentials["account"],
        )

    def role(self, connection: Connection) -> None:
        """check and set role"""
        if self._role:
            logger.info(f"Role was provided in arguments: {self._role}")
            _use(connection, UseResource.ROLE, self._role)
            return

        current = _scalar(connection, "SELECT CURRENT_ROLE()")
        if current and current.upper() != PUBLIC_ROLE:
            logger.info(f"Role is already set: {current}")
            return

        role = _first_non_public_role(connection)
        logger.info(f"Using first available role: {role}")
        _use(connection, UseResource.ROLE, role)

    def warehouse(self, connection: Connection):
        """check and set warehouse"""
        if self._warehouse:
            logger.info(
                f"Warehouse was provided in arguments: {self._warehouse}",
            )
            _use(connection, UseResource.WAREHOUSE, self._warehouse)
            return

        current = _scalar(connection, "SELECT CURRENT_WAREHOUSE()")
        if current:
            logger.info(f"Warehouse is already set: {current}")
            return

        warehouse = _scalar(connection, "SHOW WAREHOUSES")
        if not warehouse:
            raise ConnectionError("no warehouse available")
        logger.info(f"Using first available warehouse: {warehouse}")
        _use(connection, UseResource.WAREHOUSE, warehouse)

    def connect(self) -> Connection:
        """enhance default behaviour to check role and warehouse"""
        connection = super().connect()
        self.role(connection)
        self.warehouse(connection)
        return connection

==> castor_extractor-0.13.0.tar.gz: castor_extractor-0.13.0/castor_extractor/warehouse/snowflake/client.py <==
import logging
import re
from enum import Enum
from typing import Optional

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from sqlalchemy.engine import Connection

from ...utils import SqlalchemyClient, to_string_array, uri_encode

SNOWFLAKE_URI = "snowflake://{user}:{password}@{account}/?application=castor"
PK_URI = "snowflake://{user}:@{account}/?application=castor"
PUBLIC_ROLE = "PUBLIC"

logger = logging.getLogger(__name__)


def _scalar(connection: Connection, query: str) -> Optional[str]:
    """returns the first column of the first row, None if no row"""
    scalar = connection.execute(query).scalar()
    return None if scalar is None else str(scalar)


class UseResource(Enum):
    """resources using the USE syntax"""

    ROLE = "ROLE"
    WAREHOUSE = "WAREHOUSE"


def _use(connection: Connection, resource: UseResource, name: str):
    """
    USE ROLE/WAREHOUSE do not accept SQL parameter substitution
        we do a sanity check prior to using it
    DON'T COPY THIS CODE FOR ANOTHER PURPOSE
    """
    target = resource.value
    if not re.fullmatch(r"[\w_-]+", name):
        raise NameError(f"{target} name invalid {name}")
    command = f'USE {target} "{name}"'
    logger.info(f"executing {command}")
    connection.execute(command)


def _first_non_public_role(connection: Connection) -> str:
    """fetch the first non public role granted to the user"""
    raw = _scalar(connection, "SELECT CURRENT_AVAILABLE_ROLES()")
    if not raw:
        raise ConnectionError("error at fetching granted roles")
    logger.info(f"available roles {raw}")
    parsed = to_string_array(raw)
    granted = {r.upper() for r in parsed} - {PUBLIC_ROLE}
    if not granted:
        raise ConnectionError("missing non public role")
    return next(iter(granted))


def _private_key(private_key: Optional[str]) -> Optional[bytes]:
    if not private_key:
        return None

    p_key = serialization.load_pem_private_key(
        bytes(private_key, "utf-8"), password=None, backend=default_backend()
    )

    pkb = p_key.private_bytes(
        encoding=serialization.Encoding.DER,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )

    return pkb


class SnowflakeClient(SqlalchemyClient):
    """snowflake client"""

    # hide information message from the connectors
    logging.getLogger("snowflake.connector").setLevel(logging.WARNING)

    def __init__(
        self,
        credentials: dict,
        warehouse: Optional[str] = None,
        role: Optional[str] = None,
    ):
        self.private_key = _private_key(credentials.get("private_key"))
        super().__init__(credentials)
        self._role = role
        self._warehouse = warehouse

    @staticmethod
    def name() -> str:
        return "Snowflake"

    def _engine_options(self, credentials: dict) -> dict:
        if not self.private_key:
            return {}
        return {
            "connect_args": {
                "private_key": self.private_key,
            }
        }

    def _build_uri(self, credentials: dict) -> str:
        if self.private_key:
            return PK_URI.format(
                account=credentials["account"],
                user=credentials["user"],
            )
        return SNOWFLAKE_URI.format(
            user=credentials["user"],
            password=uri_encode(credentials["password"]),
            account=credentials["account"],
        )

    def role(self, connection: Connection) -> None:
        """check and set role"""
        if self._role:
            logger.info(f"Role was provided in arguments: {self._role}")
            _use(connection, UseResource.ROLE, self._role)
            return

        current = _scalar(connection, "SELECT CURRENT_ROLE()")
        if current and current.upper() != PUBLIC_ROLE:
            logger.info(f"Role is already set: {current}")
            return

        role = _first_non_public_role(connection)
        logger.info(f"Using first available role: {role}")
        _use(connection, UseResource.ROLE, role)

    def warehouse(self, connection: Connection):
        """check and set warehouse"""
        if self._warehouse:
            logger.info(
                f"Warehouse was provided in arguments: {self._warehouse}",
            )
            _use(connection, UseResource.WAREHOUSE, self._warehouse)
            return

        current = _scalar(connection, "SELECT CURRENT_WAREHOUSE()")
        if current:
            logger.info(f"Warehouse is already set: {current}")
            return

        warehouse = _scalar(connection, "SHOW WAREHOUSES")
        if not warehouse:
            raise ConnectionError("no warehouse available")
        logger.info(f"Using first available warehouse: {warehouse}")
        _use(connection, UseResource.WAREHOUSE, warehouse)

    def connect(self) -> Connection:
        """enhance default behaviour to check role and warehouse"""
        connection = super().connect()
        self.role(connection)
        self.warehouse(connection)
        return connection
